@@ -3,7 +3,10 @@ import boto.ec2
 import aws
 import netaddr
 import time
-from utils import property_lazy, MappingCustom
+from utils import property_lazy
+
+import logging
+log = logging.getLogger('motorboto.ec2')
 
 CLOUDINIT_USER_DATA = """#cloud-config
 mounts:
@@ -11,33 +14,85 @@ mounts:
         
         """
 
-
 class Connection(object):
+    """
+    Connect to EC2 through this object, and get a region from this object to set up servers, security groups etc in that region.
+    
+    >>> ec2 = motorboto.AWS().ec2
+    >>> r = ec2.regions['us-west-1']
+    >>> r.name
+    'us-west-1'
+    """
+    
     def __init__(self, aws):
         self.aws = aws
 
     @property_lazy
     def regions(self):
         c = self
-        class RegionsMapping(MappingCustom):
+        class RegionsDict(dict):
             def __getitem__(self, key):
-                return Region(key, c)
-
-            @property_lazy
-            def data(self):
-                return [r.name for r in boto.ec2.regions()]
-
-            def refresh():
-                del self.data
-
-        return RegionsMapping()
-
+                return Region(c, key)
+                
+            def __setitem__(self, key, val):
+                raise ValueError
+        return RegionsDict({ r.name: Region(c, r) for r in boto.ec2.regions() })
+    
     def _connect(self, region_name):
         return boto.ec2.connect_to_region(region_name, **self.aws._auth_params)
 
 class Region(object):
-    #contains a region connection
-    def __init__(self, name, connection):
+    """
+    Connect to EC2 through this object, and get a region from this object to set up servers, security groups etc in that region.
+    
+    >>> ec2 = motorboto.AWS().ec2
+    >>> r = ec2.regions['us-east-1']
+    
+    Here's how to easily create a new security group and key pair for a new instance, and get that instance running.
+    
+    Security group first:
+    
+    >>> sg = r.security_groups['test-sg']
+    
+    >>> import netaddr
+    >>> sg.rules[('icmp',-1,-1)] = [netaddr.IPNetwork('8.8.8.8/32')]
+    >>> sg.save()
+    
+    Then create a new key pair:
+    
+    >>> import os, shutil
+    >>> save_key_in_folder = '.afolderforyournewkey'
+    >>> os.mkdir(save_key_in_folder)
+    
+    >>> kp = r.new_key_pair('example_keypair', save_key_in_folder)
+    
+    Now we can boot our instance. Motorboto sets up instances with the latest ubuntu release by default.
+    
+    >>> i = r.new_instance('my_shiny_new_instance', sg, kp)
+    
+    >>> i.state
+    u'pending'
+    >>> i.wait('running')
+    >>> i.state
+    u'running'
+    
+    Sweet!
+    
+    >>> i.terminate()
+    >>> i.state
+    u'shutting-down'
+    
+    >>> i.wait('terminated')
+    >>> i.state
+    u'terminated'
+    
+    >>> sg.delete()
+    >>> kp.delete()
+    >>> shutil.rmtree(save_key_in_folder)
+    
+    """
+    
+    def __init__(self, connection, name):
         self.name = name
         self.connection = connection
 
@@ -63,9 +118,7 @@ class Region(object):
     def instances(self):
         return [Instance(self, i.id, i) for i in self._instances()]
         
-    def state(self):
-        return self.boto.state
-
+        
     def find_instances(self, instance_id=None, security_group_name=None, state=None):
         instances = self._instances()
         if instance_id:
@@ -83,7 +136,7 @@ class Region(object):
         return { kp.name: KeyPair(self, kp.name, kp) 
                     for kp in self.boto.get_all_key_pairs() }
         
-    def new_key_pair(self, name, save_folder):
+    def new_key_pair(self, name, save_folder=None):
         boto_key_pair = self.boto.create_key_pair(name)
         
         if save_folder:
@@ -103,7 +156,7 @@ class Region(object):
                 wait=False):
         if not ami:
             import constants
-            ami = constants.UBUNTU_AMIS['quantal'][self.name]
+            ami = constants.UBUNTU_AMIS[constants.UBUNTU_LATEST][self.name]
         
         #custom ebs size and termination policy
         import boto.ec2.blockdevicemapping
@@ -176,16 +229,34 @@ class Instance(object):
         boto_sg = self.boto.groups[0]
         return SecurityGroup(self.region, boto_sg.name, boto_sg)
         
-class Volume(object):
-    def take_snapshot(self):
-        pass
-
-class Snapshot(object):
-    @property
-    def volume(self):
-        return None
 
 class SecurityGroup(object):
+    """
+    Provides a nicer interface for working with EC2 security groups.
+    
+    >>> import netaddr
+    >>> ec2 = motorboto.AWS().ec2
+    >>> r = ec2.regions['us-east-1']
+    
+    >>> sg = r.security_groups['test-sg']
+    
+    Rules are in the form of tuples, ie (protocol, from_port, to_port)
+    For icmp, use -1
+    
+    You can allow network addresses/ranges, and also other security groups
+    
+    >>> sg.rules[('icmp',-1,-1)] = [netaddr.IPNetwork('8.8.8.8/32')]
+    >>> sg.save()
+    
+    >>> other_sg = r.security_groups['other-sg']
+    >>> other_sg.rules[('tcp',80,80)] = [netaddr.IPNetwork('8.8.8.8/32'), sg]
+    >>> other_sg.save()
+    
+    >>> other_sg.delete()
+    >>> sg.delete()
+    
+    """
+
     def __init__(self, region, name, boto=None):
         self.region = region
         self.name = name
@@ -210,7 +281,7 @@ class SecurityGroup(object):
         except KeyError:
             import datetime
             d = datetime.datetime.utcnow()
-            description = "Created by motorboto on %s" % d
+            description = "Created using motorboto on %s" % d
             return self.region.boto.create_security_group(self.name, description)
 
     @property 
@@ -269,6 +340,17 @@ class SecurityGroup(object):
 
         class SecurityGroupRulesMapping(dict):
             def __setitem__(self, key, value):
+                if not len(key) == 3:
+                    raise ValueError
+                    
+                proto, from_port, to_port = key
+                
+                if not proto in ['tcp', 'udp', 'icmp']:
+                    raise ValueError
+                
+                if proto == 'icmp' and (from_port != -1 or to_port != -1):
+                    raise ValueError
+                
                 super(SecurityGroupRulesMapping, self).__setitem__(key, set(value))
             def save(self):
                 existing = SecurityGroup(sg.region, sg.name).rules
